/*
 * ion/query/wmessage.c
 *
 * Copyright (c) Tuomo Valkonen 1999-2003. 
 * See the included file LICENSE for details.
 */

#include <string.h>

#include <ioncore/common.h>
#include <ioncore/drawp.h>
#include <ioncore/objp.h>
#include <ioncore/font.h>
#include <ioncore/global.h>
#include <ioncore/event.h>
#include "wmessage.h"
#include "inputp.h"



/*{{{ Sizecalc */


static void get_geom(WMessage *wmsg, DrawInfo *dinfo, bool max)
{
	if(max){
		dinfo->geom.w=wmsg->input.max_geom.w;
		dinfo->geom.h=wmsg->input.max_geom.h;
	}else{
		dinfo->geom.w=REGION_GEOM(wmsg).w;
		dinfo->geom.h=REGION_GEOM(wmsg).h;
	}
	dinfo->geom.x=0;
	dinfo->geom.y=0;
}


static void setup_wmsg_dinfo(WMessage *wmsg, DrawInfo *dinfo, bool max)
{
	setup_input_dinfo((WInput*)wmsg, dinfo);
	get_geom(wmsg, dinfo, max);
}


static void wmsg_calc_size(WMessage *wmsg, WRectangle *geom)
{
	WRectangle max_geom=*geom;
	DrawInfo dinfo_;
	int h;
	
	setup_wmsg_dinfo(wmsg, &dinfo_, TRUE);
	fit_listing(&dinfo_, &(wmsg->listing));
	
	h=INPUT_BORDER_SIZE(dinfo_.grdata)+wmsg->listing.toth;
	
	if(h>max_geom.h)
		h=max_geom.h;
	
	geom->h=h;
	geom->w=max_geom.w;
	geom->y=max_geom.y+max_geom.h-geom->h;
	geom->x=max_geom.x;
}


/*}}}*/


/*{{{ Draw */


static void wmsg_draw(WMessage *wmsg, bool complete)
{
	DrawInfo dinfo_;
	setup_wmsg_dinfo(wmsg, &dinfo_, FALSE);
	draw_listing(&dinfo_, &(wmsg->listing), complete);
}


/*}}}*/


/*{{{ Scroll */


static void wmsg_scrollup(WMessage *wmsg)
{
	if(scrollup_listing(&(wmsg->listing)))
		wmsg_draw(wmsg, TRUE);
}


static void wmsg_scrolldown(WMessage *wmsg)
{
	if(scrolldown_listing(&(wmsg->listing)))
		wmsg_draw(wmsg, TRUE);
}


/*}}}*/


/*{{{ Init, deinit draw config update */


static bool wmsg_init(WMessage *wmsg, WWindow *par, WRectangle geom,
					  const char *msg)
{
	char **ptr;
	int k, n=0;
	char *cmsg;
	const char *p;
	size_t l;
	
#if 0
	cmsg=scopy(msg);
	
	if(cmsg==NULL){
		warn_err();
		return FALSE;
	}
	
	ptr=ALLOC_N(char*, 1);
	if(ptr==NULL){
		warn_err();
		free(cmsg);
		return FALSE;
	}
	ptr[0]=cmsg;
	n=1;
#else	
	p=msg;
	while(p!=NULL){
		n=n+1;
		p=strchr(p+1, '\n');
	}
	
	if(n==0)
		return FALSE;
		
	ptr=ALLOC_N(char*, n);
	
	if(ptr==NULL){
		warn_err();
		return FALSE;
	}
	
	for(k=0; k<n; k++)
		ptr[k]=NULL;
	
	p=msg;
	k=0;
	while(1){
		assert(k<n);
		l=strcspn(p, "\n");
		cmsg=ALLOC_N(char, l+1);
		if(cmsg==NULL){
			while(k>0){
				k--;
				free(ptr[k]);
			}
			free(ptr);
			return FALSE;
		}
		strncpy(cmsg, p, l);
		cmsg[l]='\0';
		ptr[k]=cmsg;
		if(p[l]=='\0')
			break;
		p=p+l+1;
		k++;
	}
#endif
	
	init_listing(&(wmsg->listing));
	setup_listing(&(wmsg->listing), INPUT_FONT(GRDATA_OF(par)), ptr, n, TRUE);
	
	if(!input_init((WInput*)wmsg, par, geom)){
		deinit_listing(&(wmsg->listing));
		return FALSE;
	}

	return TRUE;
}


WMessage *create_wmsg(WWindow *par, WRectangle geom, const char *msg)
{
	CREATEOBJ_IMPL(WMessage, wmsg, (p, par, geom, msg));
}


static void wmsg_deinit(WMessage *wmsg)
{
	if(wmsg->listing.strs!=NULL)
		deinit_listing(&(wmsg->listing));

	input_deinit((WInput*)wmsg);
}


static void wmsg_draw_config_updated(WMessage *wmsg)
{
	listing_set_font(&(wmsg->listing), INPUT_FONT(GRDATA_OF(wmsg)));
	input_draw_config_updated((WInput*)wmsg);
}


/*}}}*/


/*{{{ Dynamic function table and class implementation */


static DynFunTab wmsg_dynfuntab[]={
	{draw_window,		wmsg_draw},
	{input_calc_size, 	wmsg_calc_size},
	{input_scrollup, 	wmsg_scrollup},
	{input_scrolldown,	wmsg_scrolldown},
	{region_draw_config_updated, wmsg_draw_config_updated},
	END_DYNFUNTAB
};


IMPLOBJ(WMessage, WInput, wmsg_deinit, wmsg_dynfuntab);

	
/*}}}*/
